/* This testcase is part of GDB, the GNU debugger.

   Copyright 2012-2025 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

#include <sys/ioctl.h>
#include <termios.h>
#include <stdio.h>

/* The purpose of this test is to verify gdb record_linux_system_call()
   recognizes the call for ioctl TCSETSW.  */

int
main(void)
{

  struct termios term;
  int result;
  int fd = 0;

  /* The test just needs to generate an ioctl call for TCSETSW to see if gdb
     record detected it or not.  Success or failure of the ioctl call is
     irrelevant.  */
  result = tcsetattr(fd, TCSADRAIN, &term);   /* TCSETSW call */
  result = 0;   /* TCSETSW called */
}
