/* This testcase is part of GDB, the GNU debugger.

 Copyright 2024-2025 Free Software Foundation, Inc.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

#include <immintrin.h>

void
ptwrite1 (int value)
{
  _ptwrite32 (value);
}

void
ptwrite2 (int value)
{
  _ptwrite32 (value);
}

int
main (void)
{
  ptwrite1 (0x42);
  ptwrite2 (0x43);

  return 0;
}
